@@ -1,12 +1,18 @@
 import copy
 import logging
 from datetime import datetime
-from operator import itemgetter
+
+import six
+from six import string_types
 
 from elasticsearch import Elasticsearch
 from elasticsearch.helpers import bulk_index
 
 from elasticutils._version import __version__  # noqa
+from elasticutils import monkeypatch
+
+
+monkeypatch.monkeypatch_es()
 
 
 log = logging.getLogger('elasticutils')
@@ -36,18 +42,16 @@ QUERY_ACTION_MAP = {
     'in': 'in',
     'term': 'term',
     'terms': 'terms',
-    'startswith': 'prefix',  # Backwards compatability
     'prefix': 'prefix',
-    'text': 'text',
-    'text_phrase': 'text_phrase',
-    'match': 'match',  # ES 0.19.9 renamed text to match
+    'match': 'match',
     'match_phrase': 'match_phrase',
     'wildcard': 'wildcard',
-    'fuzzy': 'fuzzy'}
+    'fuzzy': 'fuzzy'
+}
 
 
-#: List of text/match actions.
-TEXTMATCH_ACTIONS = ['text', 'text_phrase', 'match', 'match_phrase']
+#: List of match actions.
+MATCH_ACTIONS = ['match', 'match_phrase']
 #: List of range actions.
 RANGE_ACTIONS = ['gt', 'gte', 'lt', 'lte']
 
@@ -87,7 +91,7 @@ def _build_key(urls, timeout, **settings):
 
     # elasticsearch allows urls to be a string, so we make sure to
     # account for that when converting whatever it is into a tuple.
-    if isinstance(urls, basestring):
+    if isinstance(urls, string_types):
         urls = (urls,)
     else:
         urls = tuple(urls)
@@ -257,7 +261,10 @@ class F(object):
     """
     def __init__(self, **filters):
         """Creates an F"""
+
         filters = filters.items()
+        if six.PY3:
+            filters = list(filters)
         if len(filters) > 1:
             self.filters = [{'and': filters}]
         else:
@@ -324,17 +331,17 @@ class Q(object):
     example::
 
         q = Q()
-        q += Q(title__text='shoes')
-        q += Q(summary__text='shoes')
+        q += Q(title__match='shoes')
+        q += Q(summary__match='shoes')
 
     creates a BooleanQuery with two `must` clauses.
 
     Example 2::
 
         q = Q()
-        q += Q(title__text='shoes', should=True)
-        q += Q(summary__text='shoes')
-        q += Q(description__text='shoes', must=True)
+        q += Q(title__match='shoes', should=True)
+        q += Q(summary__match='shoes')
+        q += Q(description__match='shoes', must=True)
 
     creates a BooleanQuery with one `should` clause (title) and two
     `must` clauses (summary and description).
@@ -390,9 +397,9 @@ def _boosted_value(name, action, key, value, boost):
     """Boost a value if we should in _process_queries"""
     if boost is not None:
         # Note: Most queries use 'value' for the key name except
-        # Text/Match queries which use 'query'. So we have to do some
+        # Match queries which use 'query'. So we have to do some
         # switcheroo for that.
-        value_key = 'query' if action in TEXTMATCH_ACTIONS else 'value'
+        value_key = 'query' if action in MATCH_ACTIONS else 'value'
         return {name: {'boost': boost, value_key: value}}
     return {name: value}
 
@@ -416,7 +423,7 @@ class PythonMixin(object):
            This does the conversion in-place!
 
         """
-        if isinstance(obj, basestring):
+        if isinstance(obj, string_types):
             if len(obj) == 26:
                 try:
                     return datetime.strptime(obj, '%Y-%m-%dT%H:%M:%S.%f')
@@ -593,53 +600,61 @@ class S(PythonMixin):
         return self._clone(next_step=('explain', value))
 
     def values_list(self, *fields):
-        """
-        Return a new S instance that returns ListSearchResults.
+        """Return a new S instance that returns ListSearchResults.
 
         :arg fields: the list of fields to have in the results.
 
-            With no arguments, returns a list of tuples of all the
-            data for that document.
+            With no arguments, passes ``fields=*`` and returns values
+            for any fields you have marked as "stored = True" for that
+            mapping.
 
-            With arguments, returns a list of tuples where the fields
-            in the tuple are in the order specified.
+            With arguments, passes those field arguments via
+            ``fields`` and returns a list of tuples with values in the
+            order specified.
 
-        For example:
+        For example (assume id, name and age are stored fields):
 
         >>> list(S().values_list())
-        [(1, 'fred', 40), (2, 'brian', 30), (3, 'james', 45)]
+        [([1], ['fred'], [40]), ([2], ['brian'], [30]), ...]
         >>> list(S().values_list('id', 'name'))
-        [(1, 'fred'), (2, 'brian'), (3, 'james')]
+        [([1], ['fred']), ([2], ['brian']), ([3], ['james'])]
         >>> list(S().values_list('name', 'id'))
-        [('fred', 1), ('brian', 2), ('james', 3)]
+        [(['fred'], [1]), (['brian'], [2]), (['james'], [3])]
 
         .. Note::
 
-           If you don't specify fields, the data comes back in an
-           arbitrary order. It's probably best to specify fields or
-           use ``values_dict``.
+            If you do not specify any fields and you have no fields
+            marked as stored, then you will get back the ``_id`` and
+            ``_type`` of each result and that's it.
 
         """
         return self._clone(next_step=('values_list', fields))
 
     def values_dict(self, *fields):
-        """
-        Return a new S instance that returns DictSearchResults.
+        """Return a new S instance that returns DictSearchResults.
 
         :arg fields: the list of fields to have in the results.
 
-            With no arguments, this returns a list of dicts with all
-            the fields.
+            With no arguments, passes ``fields=*`` and returns values
+            for any fields you have marked as "stored = True" for that
+            mapping.
 
-            With arguments, it returns a list of dicts with the
-            specified fields.
+            With arguments, passes those field arguments via
+            ``fields`` and returns a list of dicts with the specified
+            fields.
 
-        For example:
+        For example (assuming id, name and age are stored):
 
         >>> list(S().values_dict())
-        [{'id': 1, 'name': 'fred', 'age': 40}, ...]
+        [{'id': [1], 'name': ['fred'], 'age': [40]}, ...]
         >>> list(S().values_dict('id', 'name'))
-        [{'id': 1, 'name': 'fred'}, ...]
+        [{'id': [1], 'name': ['fred']}, ...]
+
+        .. Note::
+
+            If you do not specify any fields and you have no fields
+            marked as stored, then you will get back the ``_id`` and
+            ``_type`` of each result and that's it.
 
         """
         return self._clone(next_step=('values_dict', fields))
@@ -703,7 +718,7 @@ class S(PythonMixin):
 
         >>> s = S().query(foo='bar')
         >>> s = S().query(Q(foo='bar'))
-        >>> s = S().query(foo='bar', bat__text='baz')
+        >>> s = S().query(foo='bar', bat__match='baz')
         >>> s = S().query(foo='bar', should=True)
         >>> s = S().query(foo='bar', should=True).query(baz='bat', must=True)
 
@@ -794,7 +809,11 @@ class S(PythonMixin):
         details on adding support for new filter types.
 
         """
-        return self._clone(next_step=('filter', list(filters) + kw.items()))
+        items = kw.items()
+        if six.PY3:
+            items = list(items)
+        return self._clone(
+            next_step=('filter', list(filters) + items))
 
     def filter_raw(self, filter_):
         """
@@ -827,8 +846,8 @@ class S(PythonMixin):
         Examples::
 
             q = (S().query(title='taco trucks',
-                           description__text='awesome')
-                    .boost(title=4.0, description__text=2.0))
+                           description__match='awesome')
+                    .boost(title=4.0, description__match=2.0))
 
 
         If the key is a field name, then the boost will apply to all
@@ -861,15 +880,15 @@ class S(PythonMixin):
         For example, if you had::
 
             qs = (S().boost(title=4.0, summary=2.0)
-                     .query(title__text=value,
-                            summary__text=value,
-                            content__text=value,
+                     .query(title__match=value,
+                            summary__match=value,
+                            content__match=value,
                             should=True))
 
 
-        ``title__text`` would be boosted twice as much as
-        ``summary__text`` and ``summary__text`` twice as much as
-        ``content__text``.
+        ``title__match`` would be boosted twice as much as
+        ``summary__match`` and ``summary__match`` twice as much as
+        ``content__match``.
 
         """
         new = self._clone()
@@ -883,10 +902,10 @@ class S(PythonMixin):
         You can demote documents that match query criteria::
 
             q = (S().query(title='trucks')
-                    .demote(0.5, description__text='gross'))
+                    .demote(0.5, description__match='gross'))
 
             q = (S().query(title='trucks')
-                    .demote(0.5, Q(description__text='gross')))
+                    .demote(0.5, Q(description__match='gross')))
 
         This is implemented using the boosting query in
         Elasticsearch. Anything you specify with ``.query()`` goes
@@ -926,7 +945,10 @@ class S(PythonMixin):
         Return a new S instance with raw facet args combined with
         existing set.
         """
-        return self._clone(next_step=('facet_raw', kw.items()))
+        items = kw.items()
+        if six.PY3:
+            items = list(items)
+        return self._clone(next_step=('facet_raw', items))
 
     def highlight(self, *fields, **kwargs):
         """Set highlight/excerpting with specified options.
@@ -944,7 +966,7 @@ class S(PythonMixin):
 
         For example::
 
-            q = (S().query(title__text='crash', content__text='crash')
+            q = (S().query(title__match='crash', content__match='crash')
                     .highlight('title', 'content'))
 
             for result in q:
@@ -956,7 +978,7 @@ class S(PythonMixin):
 
         For example, this search won't highlight anything::
 
-            q = (S().query(title__text='crash')
+            q = (S().query(title__match='crash')
                     .highlight('title')          # highlights 'title' field
                     .highlight(None))            # clears highlight
 
@@ -1074,7 +1096,7 @@ class S(PythonMixin):
             if action == 'order_by':
                 sort = []
                 for key in value:
-                    if isinstance(key, basestring) and key.startswith('-'):
+                    if isinstance(key, string_types) and key.startswith('-'):
                         sort.append({key[1:]: 'desc'})
                     else:
                         sort.append(key)
@@ -1158,10 +1180,10 @@ class S(PythonMixin):
             elif pq:
                 qs['query'] = pq
 
-        if as_list and list_fields:
-            fields = qs['fields'] = list(list_fields)
-        elif as_dict and dict_fields:
-            fields = qs['fields'] = list(dict_fields)
+        if as_list:
+            fields = qs['fields'] = list(list_fields) if list_fields else ['*']
+        elif as_dict:
+            fields = qs['fields'] = list(dict_fields) if dict_fields else ['*']
         else:
             fields = set()
 
@@ -1191,7 +1213,7 @@ class S(PythonMixin):
         if explain:
             qs['explain'] = True
 
-        for suggestion, (term, kwargs) in suggestions.iteritems():
+        for suggestion, (term, kwargs) in six.iteritems(suggestions):
             qs.setdefault('suggest', {})[suggestion] = {
                 'text': term,
                 'term': {
@@ -1226,7 +1248,10 @@ class S(PythonMixin):
                     continue
 
             elif isinstance(f, dict):
-                key = f.keys()[0]
+                if six.PY3:
+                    key = list(f.keys())[0]
+                else:
+                    key = f.keys()[0]
                 val = f[key]
                 key = key.strip('_')
 
@@ -1433,7 +1458,7 @@ class S(PythonMixin):
 
         if self.type is not None:
             indexes = self.type.get_index()
-            if isinstance(indexes, basestring):
+            if isinstance(indexes, string_types):
                 indexes = [indexes]
             return indexes
 
@@ -1752,7 +1777,7 @@ class SearchResults(object):
 
     Example::
 
-        s = S().query(bio__text='archaeologist')
+        s = S().query(bio__match='archaeologist')
         results = s.execute()
 
         # Shows how long the search took
@@ -1798,10 +1823,29 @@ class DictSearchResults(SearchResults):
     dict.
     """
     def set_objects(self, results):
-        key = 'fields' if self.fields else '_source'
-        self.objects = [decorate_with_metadata(DictResult(r[key]), r)
+        def listify(d):
+            return dict([(key, val if isinstance(val, list) else [val])
+                         for key, val in d.items()])
+
+        if results:
+            if 'fields' in results[0]:
+                objs = [(r['fields'], r) for r in results]
+
+            elif '_source' in results[0]:
+                objs = [(r['_source'], r) for r in results]
+
+            else:
+                # No fields and no source, so we just return _id and
+                # _type.
+                objs = [({'_id': r['_id'], '_type': r['_type']}, r)
                         for r in results]
 
+        else:
+            objs = []
+
+        # Decorate with metadata and listify values
+        self.objects = [decorate_with_metadata(DictResult(listify(obj)), r)
+                        for obj, r in objs]
 
 class ListSearchResults(SearchResults):
     """
@@ -1809,19 +1853,26 @@ class ListSearchResults(SearchResults):
     tuple.
     """
     def set_objects(self, results):
-        if self.fields:
-            getter = itemgetter(*self.fields)
-            objs = [(getter(r['fields']), r) for r in results]
+        def listify(values):
+            return [(val if isinstance(val, list) else [val])
+                    for val in values]
 
-            # itemgetter returns an item--not a tuple of one item--if
-            # there is only one thing in self.fields. Since we want
-            # this to always return a list of tuples, we need to fix
-            # that case here.
-            if len(self.fields) == 1:
-                objs = [((obj,), r) for obj, r in objs]
+        if results:
+            if 'fields' in results[0]:
+                objs = [(r['fields'].values(), r) for r in results]
+
+            elif '_source' in results[0]:
+                objs = [(r['_source'].values(), r) for r in results]
+
+            else:
+                # No fields and no source, so we just return _id and
+                # _type.
+                objs = [((r['_id'], r['_type']), r) for r in results]
         else:
-            objs = [(r['_source'].values(), r) for r in results]
-        self.objects = [decorate_with_metadata(TupleResult(obj), r)
+            objs = []
+
+        # Decorate with metadata and listify values
+        self.objects = [decorate_with_metadata(TupleResult(listify(obj)), r)
                         for obj, r in objs]
 
 
@@ -2001,7 +2052,7 @@ class MappingType(object):
             # We want instance/class attributes to take precedence.
             # So if something like that exists, we raise an
             # AttributeError and Python handles it.
-            raise AttributeError
+            raise AttributeError(name)
 
         if name == 'object':
             # 'object' is lazy-loading. We don't do this with a
@@ -2013,7 +2064,7 @@ class MappingType(object):
         if name in self._results_dict:
             return self._results_dict[name]
 
-        raise AttributeError
+        raise AttributeError(name)
 
     # Simulate read-only container access
 
